"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ono_1 = require("ono");
const worker_threads_1 = require("worker_threads");
const clone_error_1 = require("../clone/clone-error");
let messageCounter = 0;
/**
 * Enhances the messaging capabilities of the `Worker` class, adding support for async reqeust/response
 * and generator semantics.
 * @internal
 */
class Messenger extends worker_threads_1.Worker {
    constructor(filename, options) {
        super(filename, options);
        this._pending = new Map();
        this._completed = [];
        this.on("message", this._handleMessage.bind(this));
    }
    /**
     * Sends a message to the `Executor`.
     */
    postMessage(message) {
        let id = ++messageCounter;
        super.postMessage({ ...message, id });
        return id;
    }
    /**
     * Sends a message and returns a promise that resolves when the `Executor` replies to the message.
     * If an error occurs while processing the message, then the promise will reject.
     */
    async postMessageAsync(message) {
        let id = this.postMessage(message);
        return this._awaitReply(id);
    }
    /**
     * Sends a message to the `Executor` and yields all replies. If an error occurs while processing
     * the message then the generator will reject.
     */
    async *postMessageWithReplies(message) {
        // Send the message and await the first reply
        let reply = await this.postMessageAsync(message);
        while (reply.type !== "finished") {
            // Immediately start waiting for the next reply,
            // in case it arrives while this generator function is paused
            let nextReply = this._awaitReply(reply.to);
            yield reply;
            reply = await nextReply;
        }
        return reply;
    }
    /**
     * Rejects all pending messages between the `Worker` and the `Executor`.
     */
    rejectAllPendingMessages(error) {
        let currentlyPending = [...this._pending.entries()];
        this._pending.clear();
        for (let [messageId, pending] of currentlyPending) {
            this._completed.push(messageId);
            pending.reject(error);
        }
    }
    /**
     * Returns a promise that resolves when the `Executor` replies to the specified message.
     * If an error occurs while processing the message, then the promise will reject.
     */
    async _awaitReply(id) {
        return new Promise((resolve, reject) => {
            this._pending.set(id, { resolve, reject });
        });
    }
    /**
     * Handles incoming messages from the `Executor`.
     */
    _handleMessage(reply) {
        try {
            // Delete the pending message, now that it's done
            let message = this._pending.get(reply.to);
            this._pending.delete(reply.to);
            if (message) {
                this._completed.push(reply.to);
                if (reply.type === "error") {
                    message.reject(clone_error_1.createError(reply.error));
                }
                else {
                    message.resolve(reply);
                }
            }
            else if (!this._completed.includes(reply.to)) {
                // The specified message ID is neither pending nor completed
                throw ono_1.ono({ messageId: reply.to }, `Invalid message ID: ${reply.to}`);
            }
        }
        catch (error) {
            // Something went wrong while handling the reply,
            // so emit the error so it can be handled appropriately
            this.emit("error", error);
        }
    }
}
exports.Messenger = Messenger;
//# sourceMappingURL=messenger.js.map